//
// nam_interface.hh : Diffusion's NAM Interface Include File
// author           : Fabio Silva
//
// Copyright (C) 2000-2002 by the University of Southern California
// $Id: nam_interface.hh,v 1.7 2002/12/06 23:42:25 fabio Exp $
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License,
// version 2, as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
//
//

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <list>

using std::list;

#include "timers.hh"

// Configures NAM workspace
#define MIN_X 100
#define MAX_X 4000
#define MIN_Y 100
#define MAX_Y 4000
#define DEFAULT_X 1200
#define DEFAULT_Y 1200
#define MIN_NODE_SIZE 10
#define MAX_NODE_SIZE 100
#define DEFAULT_NODE_SIZE 50

#define BUFFER_SIZE 512
#define FAIL -1
#define DUMMY_TIMER 1
#define LABEL_TIMER 2
#define DUMMY_EVENT_PERIOD   100 // in msec
#define LABEL_EVENT_PERIOD 10000 // in msec

enum msg_names{
  INTEREST = 0,
  POSITIVE_REINFORCEMENT,
  NEGATIVE_REINFORCEMENT,
  PUSH_EXPLORATORY_DATA,
  EXPLORATORY_DATA,
  DATA
};

class NodeInfo{
public:
  int node_id_;
  int node_x_;
  int node_y_;
  char *node_color_;
  char *node_shape_;
  int packets_received_;
  int packets_sent_;
  bool initialized_;

  NodeInfo(int node_id, int node_x, int node_y, char *node_color, char *node_shape) :
    node_id_(node_id), node_x_(node_x), node_y_(node_y),
    node_color_(strdup(node_color)), node_shape_(strdup(node_shape))
  {
    packets_received_ = 0;
    packets_sent_ = 0;
    initialized_ = false;
  }
};

typedef list<NodeInfo *> NodeList;

class NamInterface{
public:
  NamInterface(int argc, char **argv);
  void run();
  int sendDummyEvent();
  int sendLabelEvent();

private:
  char buf[BUFFER_SIZE];
  struct timeval tv, zero_;
  bool initialized_;
  TimerManager *timers_manager_;
  NodeList nodes_;
  int packets_in_;
  int packets_out_;
  int labels_;
  int node_size_;
  int size_x_, size_y_;
  char *config_file_;

  // Node state management
  void addNode(NodeInfo *node);
  NodeInfo *findNode(int id);

  // Auxiliary functions
  float getCurrentTime();

  // Packet events
  void sendPacket(int src, int dst);

  // Nam Configuration
  void readConfigFile();
  void configNam();
  void usage(char *s);

  // Events management
  void processEvent();
  void processDiffusionLogEvent(int node_id, bool sending, bool broadcast,
				int len, int extra, int msg_type,
				double event_time);
};

class SendDummyEventTimer : public TimerCallback {
public:
  SendDummyEventTimer(NamInterface *app) : app_(app) {};
  ~SendDummyEventTimer() {};
  int expire();

  NamInterface *app_;
};

class SendLabelEventTimer : public TimerCallback {
public:
  SendLabelEventTimer(NamInterface *app) : app_(app) {};
  ~SendLabelEventTimer() {};
  int expire();

  NamInterface *app_;
};
