// 
// udp_reader.cc : Receives data on an udp socket
// authors       : Fabio Silva, John Heidemann
//
// Copyright (C) 2000-2002 by the University of Southern California
// $Id: udp_reader.cc,v 1.2 2002/08/29 01:38:44 fabio Exp $
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License,
// version 2, as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
//
//

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <netdb.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>

#define BUFFER_SIZE 512

void reader(int port)
{
  struct hostent *host_ent = NULL;
  int fd;
  struct sockaddr_in skt_addr_inet;
  int status;
  fd_set fds;
  char buf[BUFFER_SIZE];

  host_ent = gethostbyname("localhost");
  if (!host_ent){
    printf("Can't get host entry !\n");
    exit(1);
  }

  fd = socket(AF_INET, SOCK_DGRAM, 0);
  if (fd < 0){
    printf("Cannot create socket !\n");
    exit(1);
  }

  bzero((char *) &skt_addr_inet, sizeof(skt_addr_inet));

  skt_addr_inet.sin_family = AF_INET;
  skt_addr_inet.sin_port = htons(port);
  skt_addr_inet.sin_addr.s_addr = htonl(INADDR_ANY);

  if (bind(fd, (struct sockaddr *) &(skt_addr_inet),
	   sizeof(skt_addr_inet)) < 0){
    printf("Can't bind to socket !\n");
    exit(1);
  }

  while(1){
    FD_ZERO(&fds);
    FD_SET(fd, &fds);

    status = select(fd+1, &fds, NULL, NULL, NULL);

    // Check if we got something
    if (status > 0){
      status = recvfrom(fd, buf, BUFFER_SIZE, 0, NULL, NULL);
      if (status == -1){
	fprintf(stderr, "Error receiving from socket !\n");
      }
      else{
	buf[status] = 0;
	printf("%s", buf);
	fflush(NULL);
      }
    }
    else
      if (status < 0)
	printf("Select returned %d !\n", status);
  }
}

main(int argc, char **argv)
{
  int port = 0;
  int c;

  while ((c = getopt(argc, argv, "p:")) != -1)
    switch (c){
    case 'p':
      port = atoi(optarg);
      break;
    }

  if (optind != argc || port == 0){
    printf("usage: %s -p port\n", argv[0]);
    exit(1);
  }

  reader(port);
}
